package com.fundi.framework.common.preferences;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.util.ArrayList;
import java.util.StringTokenizer;

import com.fundi.framework.common.nl1.Messages;
import com.fundi.framework.common.util.Collator;

public class PreferenceCategory implements Comparable {

	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$
	
	private Object editor = null;
	private String category = null;
	private String categoryPath = null;
	private boolean hasSubCategories = false;
	
	public static final String subCategoryDelimiter = "\t"; //$NON-NLS-1$
	
	public PreferenceCategory() {
		
		super();
	}

	public PreferenceCategory(String categoryPath, Object editor) {
		
		super();

		StringTokenizer parser = new StringTokenizer(categoryPath, subCategoryDelimiter);
		
		this.categoryPath = categoryPath;
		
		while (parser.hasMoreTokens()) {
			category = parser.nextToken();  //End up with the last one.
		}

		this.editor = editor;
	}
	
	public ArrayList<PreferenceCategory> getHigherLevelCategories() {
		
		ArrayList<PreferenceCategory> higherCategories = new ArrayList<PreferenceCategory>();
		int delimPos;
		
		/*
		 *  We add a category for each level in the full category path.
		 *  
		 *  If a category already exists, its editor isn't overwritten as the first editor is the one that gets used and we don't want
		 *  to nullify an editor already supplied. 
		 */
		delimPos = categoryPath.indexOf(PreferenceCategory.subCategoryDelimiter);
		while (delimPos > -1) {
			higherCategories.add(new PreferenceCategory(categoryPath.substring(0, delimPos), null));
			delimPos = categoryPath.indexOf(PreferenceCategory.subCategoryDelimiter, delimPos + 1);
		}
		
		return higherCategories;
	}

	@Override
	public int compareTo(Object arg0) {
		
		return (new Collator()).compare(getCategoryPath(), ((PreferenceCategory) arg0).getCategoryPath());
	}

	public Object getEditor() {
		
		return editor;
	}

	public void setEditor(Object editor) {
		
		this.editor = editor;
	}

	public boolean hasSubCategories() {
		
		return hasSubCategories;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((categoryPath == null) ? 0 : categoryPath.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		PreferenceCategory other = (PreferenceCategory) obj;
		if (categoryPath == null) {
			if (other.categoryPath != null)
				return false;
		} else if (!categoryPath.equals(other.categoryPath))
			return false;
		return true;
	}

	public void setHasSubCategories(boolean hasSubCategories) {
		
		this.hasSubCategories = hasSubCategories;
	}

	public String getCategory() {
		
		return category;
	}

	public String getCategoryPath() {
		
		return categoryPath;
	}
}